
/*
 *
 *   Copyright (c) Rocket Division Software 2001-2010. All rights reserved.
 *   Copyright (c) StarBurn Software 2001-2010. All rights reserved.
 *
 */

using System;
using System.Text;
using System.Drawing;
using System.Threading; 
using System.Collections;
using System.Windows.Forms;
   
using RocketDivision.StarBurnX;

namespace DataBurnerNS
{
	public class frmDataBurner : Form
	{
		// * | Declarations | *
		#region Constants
        
		private const string DIRECTORY_NODE_KEY     = "$D";
		private const string ROOT_NODE_KEY          = "$R";
		private const string DISC_LABEL_PREFIX      = "Disc :";
		private const string DEF_VOLUME_NAME        = "DATA_DISC";
		private const int    DEF_PROGRESS_CAPACITY  = 32000;
		private const int    MODE1_LB_SIZE          = 2048;  // This constant is used to calculate size of the Data track (in kb.)
        
		#endregion Constants

		#region Members

		private StarBurnX           m_starBurnX;
		private UDFDataBurner       m_dataBurner;
		private Drives              m_drivesBuff;
		private DriveSpeeds         m_driveWriteSpeeds;
		
		private long                m_curSessionSize;   // In kb
		private long                m_prevSessionSize;  // In kb
		private long                m_totalDiscCapacity;// In kb

		private frmProgress         m_burningProgressForm;
		private frmFSAddingProgress m_fsAddingProgressForm;
        private frmValidation       m_validatingForm;    

		private string			    m_volumeName;
		private ArrayList           m_recordableDevices             = new ArrayList(); 
        private bool                m_isSessionImported             = false;

        private bool                m_crossThreadParam_IsTestMode   = false;
        private IDataFolder         m_crossThreadParam_CurFolder;
        private IDataFolder         m_crossThreadParam_AddedFolder;
        private string              m_crossThreadParam_AddingFolderPath;   
 
        private bool                m_isDiscBurned                  = false;        
        #endregion Members

		#region Delegates
        
		public delegate void FSOperationHandler();
        public delegate void OperationHandler  ();
		public delegate void StopProgress      ();

		#endregion

		// * | Implementation | *

		#region Construction
		public frmDataBurner()
		{
			InitializeComponent();

			m_volumeName = DEF_VOLUME_NAME;
		}

		#endregion Construction

		#region StarBurnX events
        
		private void OnTestWriteDisabled(ref bool CancelBurning)
		{
			if (MessageBox.Show("The current disc does not support the Test write mode! Do you want to continue the *REAL* burning?", "", MessageBoxButtons.YesNo, MessageBoxIcon.Information ) == DialogResult.No  )
			{
				CancelBurning = true;
			}
        }

		public STARBURN_NAME_COLLISION_SOLUTION OnNameCollision(string ExistingFilePath, string AddingFilePath, ref string AddingFileName)
		{
			return STARBURN_NAME_COLLISION_SOLUTION.STARBURN_NAME_COLLISION_RENAME_NEW;
		}

		public void OnProgress(int Percent, int TimeRemaining)
		{
			if (m_burningProgressForm != null)
			{
				m_burningProgressForm.SetBurningProgress(Percent);  
			}
			else if(m_fsAddingProgressForm != null)
			{
				m_fsAddingProgressForm .SetProgress(Percent); 
			}
		}

		public void OnBurningBuffer(int Percent)
		{
			if (m_burningProgressForm!= null)
			{
				m_burningProgressForm.SetBurningBuffer(Percent); 
			}
		}

		#endregion StarBurnX events

		#region Burning/Adding FS

		protected void DoAppendDirectory()
		{
            try
            {
                m_crossThreadParam_AddedFolder = m_crossThreadParam_CurFolder.AddDirectory(m_crossThreadParam_AddingFolderPath);
            }
            catch
            {
            }
            finally
            {
                OnDirectoryAdded(); 
            }
		}

		protected void DoBurn()
		{
			// Do burning
			try
			{
                m_dataBurner.Burn(m_crossThreadParam_IsTestMode, m_volumeName);
                //m_dataBurner.Burn(m_crossThreadParam_IsTestMode, m_volumeName, "StarBurnX", "StarBurnX Data Burner");
			}
			catch
			{

			}
			finally
			{
				OnStopBurningProgress(); 
			}
		}

		#endregion

		#region Progress
        
		protected void OnDirectoryAdded()
		{
			if(!this.InvokeRequired )
			{
				OnStopFSAddingProgress();
                
                if (m_crossThreadParam_AddedFolder != null)
                {
                    // Expand cuurent folder
                    treeDiscContent.SelectedNode.Expand();

                    // Append item into tree
                    treeDiscContent.SelectedNode = AddFileIntoTree(treeDiscContent.SelectedNode, m_crossThreadParam_AddedFolder, false);

                    RefreshTotalSessionsSize();
                }
                else
                {
                    if ( m_fsAddingProgressForm != null && m_fsAddingProgressForm.IsCanceled ( ) )
                    {
                        ShowErrorMessageBox (
                                "operation has been canceled by user!" );
                    }
                    else
                    {
                        if (m_crossThreadParam_CurFolder.LastErrorCode ==
                            STARBURN_RESULT_CODES.STARBURN_ERR_INCORRECT_IMAGE_STATE)
                        {
                            ShowErrorMessageBox(
                                "Adding error! Internal file system is alredy built and You can not modify tree!");
                        }
                        else
                        {
                            ShowErrorMessageBox("Adding error!");
                        }
                    }
                }

                btnImportTrack.Enabled = false;
				btnClear.Enabled	   = true;

				Activate();

                m_crossThreadParam_AddedFolder  = null;
                m_crossThreadParam_CurFolder    = null;
			}
			else
			{
				FSOperationHandler asynFSOperation = new FSOperationHandler(OnDirectoryAdded);
                
				this.Invoke(asynFSOperation);
			}
		}

        private void DoVerify()
        {
            try
            {
                m_dataBurner.Verify(); 
            }
            catch
            {
            }
            finally
            {
                OnStopVerification();
            }
        }

        private void OnStopVerification()
        {
            if (!this.InvokeRequired)
            {
                // Hide the validation window
                if (m_validatingForm != null)
                {
                    m_validatingForm.Hide(); 
                    m_validatingForm.Dispose();
                    m_validatingForm = null;

                    if (this.m_dataBurner.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_OPERATION_SUCCEEDED)
                    {
                        MessageBox.Show("Verification complete!", "Verification", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        MessageBox.Show("Disc has been successfully burned.", "Burning", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                    else
                    {
                        ShowErrorMessageBox("Verification process has been failed!"); 
                    }
                }

                m_dataBurner.LockTray = false;

                GetCurrentDrive().Eject();
                
                Enabled = true;
            }
            else
            {
                OperationHandler asynOperation = new OperationHandler(OnStopVerification);

                this.Invoke(asynOperation);
            }
            
        }

		private void OnStopBurningProgress()
		{
            if(!this.InvokeRequired )
            {
                m_isDiscBurned              = true;
                bool isOperationCanceled    = m_burningProgressForm.IsCanceled();
				
				RefreshButtonsStatus(); 

                // Hide the progress window
                if(m_burningProgressForm != null)
                {
                    m_burningProgressForm.Hide(); 
					m_burningProgressForm.Dispose();
					m_burningProgressForm = null;
                }

                if (isOperationCanceled)
                {
                    MessageBox.Show("Burning of the disc has been canceled by user.", "Burning", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    if (m_dataBurner.LastErrorCode != STARBURN_RESULT_CODES.STARBURN_OPERATION_SUCCEEDED)
                    {
                        if (m_dataBurner.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_ERR_WRONG_LBA_ADDRESS)
                        {
                            // We suppressed the standrt error message and provide the user friendly message
                            ShowErrorMessageBox("The wrong starting LB address!\n" +
                                                "This error means that you are trying to burn data on the same disc again or the new burning disc is not empty.\n" +
                                                "If you want to burn your file structure on more one disc you can do it on the empty discs only!\n" +
                                                "\n" +
                                                "Note also that you can not modify the disc file structure after the DataBurner has built the files tree for burning. So you should create the new DataBurner instance and try to make the new disc file structure.");

                        }
                        else if (m_dataBurner.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_ERR_DISC_FINALIZED)
                        {
                            ShowErrorMessageBox("Burning operation can not be performed because disc is finalized!");
                        }
                        else
                        {
                            ShowErrorMessageBox("Error has been occured during of the process! \n" + m_dataBurner.LastError);
                        }

                        m_dataBurner.LockTray = false;

                        GetCurrentDrive().Eject();

                        Enabled = true;
                    }
                    else
                    {
                        if (chkVerifyData.Checked && !chkTestMode.Checked)
                        {
                            // Create the validation window
                            m_validatingForm = new frmValidation();
                            AddOwnedForm(m_validatingForm);

                            // Show the validation window
                            m_validatingForm.Show();
         
                            // Start validation
                            Thread validationThread = new Thread(new ThreadStart(this.DoVerify));
                            validationThread.Start();		
                        }
                        else
                        {
							if (m_dataBurner.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_OPERATION_SUCCEEDED)
							{
								MessageBox.Show("Disc has been successfully burned.", "Burning", MessageBoxButtons.OK, MessageBoxIcon.Information);
							}
							else
							{
								MessageBox.Show("Burning failed.", "Burning", MessageBoxButtons.OK, MessageBoxIcon.Information);
							}

                            m_dataBurner.LockTray = false;

                            GetCurrentDrive().Eject();

                            Enabled = true;
                        }
                    }
                }
            }
            else
            {
                OperationHandler asynOperation = new OperationHandler(OnStopBurningProgress);
                
                this.Invoke(asynOperation);
            }
		}

		private void OnStopFSAddingProgress()
		{
            if(!this.InvokeRequired )
            {
                if(m_fsAddingProgressForm != null)
                {
                    m_fsAddingProgressForm.Hide(); 
					m_fsAddingProgressForm.Dispose();
                    m_fsAddingProgressForm = null;
                }
            
                m_dataBurner.LockTray  = false;
            
                Enabled = true;
            }
            else
            {
                OperationHandler asynOperation = new OperationHandler(OnStopFSAddingProgress);
                
                this.Invoke(asynOperation);
            }
		}

		#endregion Progress

		#region Common
        
		private TreeNode AddFileIntoTree(TreeNode pParentNode, IDataFile pFileObject, bool isPrevSessionFile)
		{
			TreeNode    addedNode;
			TreeNode    insertAfterNode = null;
			String      folderName = pFileObject.Name;
            
			DataFiles   subItems        = null;
            
			foreach(TreeNode    node in pParentNode.Nodes)
			{
				if( IsNodeFolder(node))
				{
					if(folderName.CompareTo(node.Text) < 0 )
					{
						insertAfterNode = node;
					}
					break;
				}
			}
            
			addedNode = new TreeNode(folderName);

			if (pFileObject is DataFolder)
			{
				DataFolder dataFolder = pFileObject as DataFolder;
                
				addedNode.Tag = DIRECTORY_NODE_KEY;
				addedNode.SelectedImageIndex = 1;
				addedNode.ImageIndex = 1;
                
				subItems = dataFolder.Files;
			}
			else
			{
				addedNode = new TreeNode(folderName);
				addedNode.SelectedImageIndex = 2;
				addedNode.ImageIndex = 2;
                
				addedNode.Tag = pFileObject.Path;
                
				// Calculate total used size
				if (!isPrevSessionFile)
				{
					m_curSessionSize = m_curSessionSize + pFileObject.Size;
				}
			}
            
			if (isPrevSessionFile)
			{
				addedNode.ForeColor = Color.Gray;
			}
			else
			{
				addedNode.ForeColor = Color.Black;
			}

			if(insertAfterNode == null)
			{
				pParentNode.Nodes.Insert(0, addedNode);
			}
			else
			{
				pParentNode.Nodes.Insert(insertAfterNode.Index, addedNode);
			}

			if (subItems != null)
			{
                
				foreach(IDataFile dataFile in subItems)
				{
					AddFileIntoTree(addedNode, dataFile, isPrevSessionFile);
				}
			}

			return addedNode;
		}

		private void RefreshTotalSize()
		{
			prgrsDiscSize.Minimum = 0;
            
			if (m_totalDiscCapacity != 0)
			{
				prgrsDiscSize.Maximum = (int)(m_totalDiscCapacity/1024);
			}
			else
			{
				prgrsDiscSize.Maximum = DEF_PROGRESS_CAPACITY;
			}
            
			lbTotalSizeValue.Text = FormatKb2String(m_totalDiscCapacity);
		}

		private void RefreshFreeDiscSize()
		{
			long freeSize;
            
			freeSize = m_totalDiscCapacity - (m_curSessionSize + m_prevSessionSize);
            
			lbFreeSpaceValue.Text = FormatKb2String(freeSize);
            
			if (freeSize >= 0)
			{
				lbFreeSpaceValue.ForeColor = Color.Black;
			}
			else
			{
				lbFreeSpaceValue.ForeColor = Color.Red;
			}
		}
        
		private void RefreshTotalSessionsSize()
		{
			lbSessionSizeValue.Text = FormatKb2String(m_curSessionSize + m_prevSessionSize);
            
			int totalSizeinMb = (int)((m_curSessionSize + m_prevSessionSize)/1024);
			if (prgrsDiscSize.Maximum > totalSizeinMb)
			{
				prgrsDiscSize.Value = totalSizeinMb;
			}
			else
			{
				prgrsDiscSize.Value = prgrsDiscSize.Maximum;
			}

			RefreshFreeDiscSize();
		}

		private void RefreshDrivesList()
		{
			if (m_drivesBuff == null)
			{
				m_drivesBuff = m_starBurnX.Drives;
			}

			cmbDrives.Items.Clear();
            cmbWriteSpeed.Items.Clear();
   
			m_recordableDevices.Clear();
	         
			int RecordableDeviceIndex   = 0;
			
			foreach (Drive drive in m_drivesBuff)
			{
				IDriveInfo driveInfo = drive.DriveInfo;

				//Check, whether the current drive is a recordable device
				if ((driveInfo.SupportedMediaWrite & (int)STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_RECORDABLE) != 0)
				{
					StringBuilder driveViewBuilder = new StringBuilder();

					driveViewBuilder.Append("[");
					driveViewBuilder.Append(driveInfo.Letter);
					driveViewBuilder.Append(":] ");
					driveViewBuilder.Append(driveInfo.Vendor);
					driveViewBuilder.Append(" ");
					driveViewBuilder.Append(driveInfo.Product);

					cmbDrives.Items.Add(driveViewBuilder.ToString());
					m_recordableDevices.Add(RecordableDeviceIndex);
				}

				RecordableDeviceIndex++;
			}

			if (cmbDrives.Items.Count == 0)
			{
				m_driveWriteSpeeds = null; 
				cmbDrives.Enabled  = false;
				cmbDrives.Items.Add("No recordable devices found");
				cmbDrives.SelectedIndex = 0; 
			}
			else
			{
				cmbDrives.Enabled = true; 
			}
		}
        
        private void RefreshButtonsStatus()
        {
            btnAddDir.Enabled		= !m_isDiscBurned;
            btnAddFiles.Enabled		= !m_isDiscBurned;
            btnImportTrack.Enabled	= !m_isDiscBurned;
            btnNewDir.Enabled		= !m_isDiscBurned;
            btnRemove.Enabled		= !m_isDiscBurned;
            btnRename.Enabled		= !m_isDiscBurned;
        }

		private void RefreshSupportedWriteSpeeds(IDriveInfo pDriveInfo)
		{
			// Clear previous values of speeds
			cmbWriteSpeed.Items.Clear();

			if(pDriveInfo == null)
			{
				cmbWriteSpeed.Enabled = false;
				cmbWriteSpeed.Items.Add("Not supported");
				return;
			}

			cmbWriteSpeed.Enabled = true;

			// Get the supported Write speeds
			m_driveWriteSpeeds = pDriveInfo.SupportedWriteSpeeds;

			foreach (DriveSpeed suppDriveSpeed in m_driveWriteSpeeds)
			{
				StringBuilder speedInfo = new StringBuilder();
                
				speedInfo.Append(suppDriveSpeed.SpeedRatio);
				speedInfo.Append("x (");
				speedInfo.Append(suppDriveSpeed.Speed);
				speedInfo.Append(" KB/sec)");

				cmbWriteSpeed.Items.Add(speedInfo.ToString());

				if (pDriveInfo.WriteSpeed == suppDriveSpeed.Speed)
				{
					cmbWriteSpeed.SelectedIndex = cmbWriteSpeed.Items.Count - 1;
				}
			}
		}

		private void RefreshBootImageSettings()
		{
			cmbBootImageType.Enabled = chkBootDisc.Checked;
			txtBootImageFile.Enabled = chkBootDisc.Checked;
			btnSelectBootImage.Enabled = chkBootDisc.Checked; 
		}

		private void InitializeBootImageControls()
		{
			// Set disc as is not bootable by default
			chkBootDisc.Checked = false;
			RefreshBootImageSettings();

			cmbBootImageType.Items.Add("CD-ROM");
			cmbBootImageType.Items.Add("Floppy");
			cmbBootImageType.Items.Add("Hard disc");

			cmbBootImageType.SelectedIndex = 0;
		}

		private void InitializeImages()
		{
            
		}

		private void InitializeDiscContentTree()
		{
            
			// Create the root node
			TreeNode rootNode = treeDiscContent.Nodes.Add(DISC_LABEL_PREFIX);

			rootNode.Tag = ROOT_NODE_KEY;

			btnRename.Enabled = false;
			btnRemove.Enabled = false;
			btnClear.Enabled  = false;

			treeDiscContent.SelectedNode = rootNode; 
                        
		}

		private bool IsNodeReadOnly(TreeNode pNode)
		{
			return pNode.ForeColor.Equals(Color.Gray);
		}
            
		private bool IsNodeFolder(TreeNode pNode)
		{
			if (pNode.Tag.ToString() == DIRECTORY_NODE_KEY)
			{
				return true;       
			}
                
			return false; 
		}

		private     Drive   GetCurrentDrive()
		{
			if (m_recordableDevices.Count == 0)
			{
				return null;
			}
            
			return m_drivesBuff[(int)m_recordableDevices[cmbDrives.SelectedIndex]];
		}
        

		private void RefreshDiscLabel()
		{
			GetNodeDiscRoot().Text = DISC_LABEL_PREFIX + m_volumeName;
		}

        
        

		#endregion Common
        
		#region Services
        
		private long ConvertLB2Kb(long lbValue) 
		{
			if (lbValue > 0)
			{
				double t;
				t = (lbValue / (1024));
				return (long)(t * MODE1_LB_SIZE);
			}
			return 0;
		}
        
		private string FormatKb2String(long kbValue) 
		{
			if (kbValue <= 0)
			{
				return "0 Mb";
			}
			else
			{
				if (kbValue < 1024)
				{
					return kbValue.ToString() + " Kb";
				}
			}                                                         
            
			return ((kbValue / 1024)).ToString() + " Mb";
		}

		private void ShowErrorMessageBox(string errorMessage)
		{
			MessageBox.Show(errorMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);  
		}

		private TreeNode GetNodeDiscRoot()
		{
			return treeDiscContent.Nodes[0];
		}
        
		private void StartLongProcess()
		{
			this.Enabled = false;
			this.Cursor  = Cursors.WaitCursor;
 
			Application.DoEvents(); 
		}

		private void EndLongProcess()
		{
			this.Enabled = true;
			this.Cursor  = Cursors.Default;

			Application.DoEvents(); 
		}

		#endregion Services

		#region Events handlers
		
		private void OnNodeSelected(TreeNode selectedNode)
		{
			if (selectedNode.Parent == null)
			{
				btnRemove.Enabled = false;
				btnRename.Enabled = false;

				if (selectedNode.Nodes.Count == 0)
				{
					btnClear.Enabled = false;
				}
				else
				{
					btnClear.Enabled = true;
				}

				btnAddDir.Enabled = btnAddFiles.Enabled = btnNewDir.Enabled = true;    
                
				return;
			}

			if (IsNodeFolder(selectedNode))
			{
				btnAddFiles.Enabled = btnAddDir.Enabled = btnNewDir.Enabled = true;
			}
			else
			{
				btnAddFiles.Enabled = btnAddDir.Enabled = btnNewDir.Enabled = false;
			}

			btnRemove.Enabled = btnRename.Enabled = true;
		}
		
		private void treeDiscContent_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
			OnNodeSelected(e.Node);
		}

		private void btnClose_Click(object sender, EventArgs e)
		{
            if (MessageBox.Show("Do You want to exit?","Exit", MessageBoxButtons.YesNo,MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }

			Application.Exit(); 
		}
        
		private void btnBurn_Click(object sender, EventArgs e)
		{
			BootImage bootImage;  
            
			// Collect information about the current selected drive
			Drive   activeDrive = GetCurrentDrive(); 
            
			if(activeDrive == null)
			{
				ShowErrorMessageBox("Drive is not specified or no recordable devices found!");  
				
				return;	
			}

			long bootImageSize = 0;

			// Collect the boot image information
			if (chkBootDisc.Checked == true)
			{
				// - Verify, whether the name of the boot image file is specified
				if (txtBootImageFile.Text.Length == 0)
				{
					ShowErrorMessageBox("Please, specify name of the boot image file.");
					txtBootImageFile.Focus();

					return;
				}
                
				if(! new System.IO.FileInfo(txtBootImageFile.Text).Exists )
				{
					ShowErrorMessageBox("The Specified image file does not exist!");
					txtBootImageFile.Focus();
				}

				bootImage = new BootImage();

				bootImage.FileName = txtBootImageFile.Text;
                
				switch(cmbBootImageType.SelectedIndex)
				{
					case 0: bootImage.Type = STARBURN_BOOT_IMAGE_TYPE.STARBURN_BOOT_IMAGE_CDROM; break;
					case 1: bootImage.Type = STARBURN_BOOT_IMAGE_TYPE.STARBURN_BOOT_IMAGE_FLOPPY;break;
					case 2: bootImage.Type = STARBURN_BOOT_IMAGE_TYPE.STARBURN_BOOT_IMAGE_HDD;   break;  
				}

				m_dataBurner.BootImage = bootImage;

				bootImageSize = (new System.IO.FileInfo(txtBootImageFile.Text).Length) / 1024;
			}

            m_dataBurner.NextSessionAllowed = chkMultiSDisc.Checked; 
			
			m_dataBurner.Drive = activeDrive;
            
			DriveInfo driveInfo = activeDrive.DriveInfo;
 
			driveInfo.Refresh() ;

			// Set the selected write speed
			if(cmbWriteSpeed.SelectedIndex >= 0 && cmbWriteSpeed.SelectedIndex < driveInfo.SupportedWriteSpeeds.Count)
			{
				m_dataBurner.WriteSpeed = driveInfo.SupportedWriteSpeeds[cmbWriteSpeed.SelectedIndex].Speed;  
			}
			else
			{
				m_dataBurner.WriteSpeed = 0;
			}

			// Test, whether drive is ready to burn
			if(activeDrive.State != STARBURN_DRIVE_STATE.STARBURN_DRIVE_STATE_READY)
			{
				ShowErrorMessageBox("Device is not ready to burn! Please make sure that tray is closed and the valid media is inserted.");  
                
				return;
			}

			// Check, whether data is specified for burning
			if(treeDiscContent.Nodes[0].Nodes.Count ==0 )
			{
				ShowErrorMessageBox("Any data for burning is not specified!");
                
				return;
			}

			DiscInfo discInfo = activeDrive.DiscInfo;
 
			discInfo.Refresh();

			// Check, whether free space is available
			if (ConvertLB2Kb(discInfo.TotalSize - discInfo.UsedSize) < (m_curSessionSize + bootImageSize ))
			{
				ShowErrorMessageBox("The disc has no free space!");
				return;
			}
            
            if (chkTestMode.Checked && !activeDrive.DriveInfo.SupportTestWrite)
            {
                if( MessageBox.Show("The current drive can not perform the test burning! Do You want to continue burning or cancel operation?", "Question", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) == DialogResult.Cancel )
                {
                    return;
                }
            }

            // Set the burn method
            m_dataBurner.Mode = STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO; 

			// Lock Tray
			m_dataBurner.LockTray = true;
            Enabled = false;
            
			// Create the progress window
			m_burningProgressForm = new frmProgress(m_dataBurner);
			AddOwnedForm(m_burningProgressForm);
            
            // Show the progress form
			m_burningProgressForm.Show();

			//Start thread
            //m_isDiscBurning = true;
            m_crossThreadParam_IsTestMode = (chkTestMode.Checked);

            Thread burningThread = new Thread(new ThreadStart(this.DoBurn));
			burningThread.Start();		
		}

		private void btnImportTrack_Click(object sender, EventArgs e)
		{
			StartLongProcess();

			btnImportTrack.Enabled = false;

			try
			{
				m_dataBurner.Drive = GetCurrentDrive();
                
                m_isSessionImported = true;

				m_dataBurner.ImportTrack(-1);

				foreach(IDataFile dataFile in m_dataBurner.Files)
				{
					AddFileIntoTree(GetNodeDiscRoot(), dataFile, true);
				}
                
				MessageBox.Show("Previous session has been successfully imported", "", MessageBoxButtons.OK, MessageBoxIcon.Information);

				btnImportTrack.Enabled = false;

			}
			catch
			{
				ShowErrorMessageBox(m_dataBurner.LastError); 
			}
                        
			EndLongProcess();
		}

		private void frmDataBurner_Load(object sender, EventArgs e)
		{
			// Create and Initialize component
			m_starBurnX = new StarBurnX();

            m_starBurnX.DebugFacility = STARBURN_DEBUG_FACILITIES.STARBURN_DEBUG_FACILITY_FILE;
            m_starBurnX.DebugFileName = @"c:\starburn_log.txt";
			m_starBurnX.Initialize(string.Empty);

			m_dataBurner = new UDFDataBurner();

            m_dataBurner.OnNameCollision += new IUDFDataBurnerEvents_OnNameCollisionEventHandler(this.OnNameCollision);
			m_dataBurner.OnProgress += new IUDFDataBurnerEvents_OnProgressEventHandler(this.OnProgress);
			m_dataBurner.OnBuffer += new IUDFDataBurnerEvents_OnBufferEventHandler(this.OnBurningBuffer);
            m_dataBurner.OnTestWriteDisabled += new IUDFDataBurnerEvents_OnTestWriteDisabledEventHandler(this.OnTestWriteDisabled);

            //m_dataBurner.OnNameCollision	 += new IDataBurnerEvents_OnNameCollisionEventHandler(this.OnNameCollision);
            //m_dataBurner.OnProgress			 += new IDataBurnerEvents_OnProgressEventHandler(this.OnProgress);
            //m_dataBurner.OnBuffer			 += new IDataBurnerEvents_OnBufferEventHandler(this.OnBurningBuffer);
            //m_dataBurner.OnTestWriteDisabled += new IDataBurnerEvents_OnTestWriteDisabledEventHandler(OnTestWriteDisabled);
		}

        STARBURN_NAME_COLLISION_SOLUTION m_dataBurner_OnNameCollision(string ExistingFilePath, string AddingFilePath, ref string AddingFileName)
        {
            throw new NotImplementedException();
        }
        
		private void frmDataBurner_Activated(object sender, System.EventArgs e)
		{
			if (m_drivesBuff == null)
			{
				Application.DoEvents(); // Redraw the form content

				InitializeBootImageControls();
				InitializeDiscContentTree();
				InitializeImages();
 
				RefreshDrivesList();

				if (cmbDrives.Items.Count > 0)
				{
					cmbDrives.Enabled = true;
					cmbDrives.SelectedIndex = 0;// Activate first drive
				}
				else
				{
					cmbDrives.Enabled = false;
				}
			}
		}

		private void frmDataBurner_Closed(object sender, System.EventArgs e)
		{
			m_driveWriteSpeeds  = null; 
			m_drivesBuff        = null;
			m_dataBurner        = null;
 
			if(m_starBurnX != null)
			{
				m_starBurnX.Uninitialize();
			}

			m_starBurnX = null;  
		}

		private void btnRefreshDrives_Click(object sender, EventArgs e)
		{
			string currentSelectedDrive = String.Empty;

			if (m_recordableDevices.Count  > 0)
			{
				// Keep current selected drive
				currentSelectedDrive = cmbDrives.Items[cmbDrives.SelectedIndex] as string;
			}
			Enabled = false;       // Lock access to the form

			m_starBurnX.Refresh();

			m_drivesBuff = m_starBurnX.Drives;

			RefreshDrivesList();

			if (m_recordableDevices.Count > 0 && currentSelectedDrive.Length > 0)
			{
				for (int spinDrives = 0; spinDrives < cmbDrives.Items.Count; spinDrives++)
				{
					if (String.Compare(currentSelectedDrive,
						cmbDrives.Items[spinDrives] as string) == 0)
					{
						cmbDrives.SelectedIndex = spinDrives;
					}
				}
			}
            
            if(cmbDrives.SelectedIndex == -1 && cmbDrives.Items.Count == 1)
            {
                cmbDrives.SelectedIndex = 0;
            }
            else
            {
                RefreshStatusImportTrack();
            }

			Enabled = true;       // Unlock access to the form
		}
        
        private void RefreshStatusImportTrack()
        {
            Drive      drive        = null;
            DiscInfo   discInfo     = null;

            drive     = GetCurrentDrive();

            btnImportTrack.Enabled = false;

            if(drive != null)
            {
                discInfo  = drive.DiscInfo;
                
                if(discInfo.Status == STARBURN_DISC_STATUS.STARBURN_DISC_STATUS_INCOMPLETE && !m_isDiscBurned  && !m_isSessionImported )
                {
                    btnImportTrack.Enabled = (discInfo.Sessions.Count > 0) ? true : false;
                }
            }
        }

		private void cmbDrives_SelectedIndexChanged(object sender, EventArgs e)
		{
			IDrive      drive        = null;
			IDriveInfo  driveInfo    = null;
			IDiscInfo   discInfo     = null;
            
			chkTestMode.Enabled = chkVerifyData.Enabled = chkMultiSDisc.Enabled = chkVerifyData.Enabled  = btnBurn.Enabled = btnImportTrack.Enabled = true;

			if (m_recordableDevices.Count > 0)
			{
				drive     = GetCurrentDrive();
				driveInfo = drive.DriveInfo;
				discInfo  = drive.DiscInfo;
			}

			RefreshSupportedWriteSpeeds(driveInfo);

			if (driveInfo != null)
			{
				chkTestMode.Enabled = driveInfo.SupportTestWrite;
			}
			else
			{
				chkTestMode.Enabled = false; 
			}

			if (discInfo != null)
			{
				m_prevSessionSize = ConvertLB2Kb(discInfo.UsedSize);
				m_totalDiscCapacity = ConvertLB2Kb(discInfo.TotalSize);
			    
                // Check, whether the disc is not closed
			    if (discInfo.Status == STARBURN_DISC_STATUS.STARBURN_DISC_STATUS_COMPLETE 
				    || 
                   (discInfo.DiscType & (int)STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_RECORDABLE) == 0)
			    {
				    chkTestMode.Enabled = chkVerifyData.Enabled = chkMultiSDisc.Enabled = chkVerifyData.Enabled  = btnBurn.Enabled = btnImportTrack.Enabled = false;
			    }
            }
			
            RefreshStatusImportTrack();

			RefreshDiscLabel();

			RefreshTotalSize();

			RefreshTotalSessionsSize();
            
		}
        
		private void chkBootDisc_CheckedChanged(object sender, EventArgs e)
		{
			RefreshBootImageSettings();           
		}

		private void btnSelectBootImage_Click(object sender, EventArgs e)
		{
			if (dlgOpenBootImageFile.ShowDialog() == DialogResult.OK)
			{
				txtBootImageFile.Text = dlgOpenBootImageFile.FileName;    
			}
		}
        
		private void btnAddFiles_Click(object sender, EventArgs e)
		{
			// Select files for adding
			try
			{
				if (dlgSelectFiles.ShowDialog() != DialogResult.OK)
				{
					return;
				}
			}
			catch
			{
				ShowErrorMessageBox("Too many files are selected!!!");
				return;
			}

			IDataFolder dataFolder;
            
			if (treeDiscContent.SelectedNode.Parent == null)
			{
				dataFolder = m_dataBurner;
			}
			else
			{
				string pathToSearch = treeDiscContent.SelectedNode.FullPath;
				dataFolder = m_dataBurner.GetFileObject(pathToSearch.Substring(pathToSearch.IndexOf('\\') + 1) + '\\') as IDataFolder; 
			}
            
            try
            {
                foreach (string fileName in dlgSelectFiles.FileNames)
                {
                    IDataFile addedFile = dataFolder.AddFileEx(fileName, @"\test.jpg");

                    if (addedFile != null)
                    {
                        // Append item into tree
                        TreeNode fileNode = treeDiscContent.SelectedNode.Nodes.Add(addedFile.Name);
                        fileNode.Tag = addedFile.Path;
                        fileNode.ImageIndex = 2;
                        fileNode.SelectedImageIndex = 2;

                        // Update total size value
                        m_curSessionSize = m_curSessionSize + addedFile.Size;
                    }
                }
            }
            catch
            {
                if(dataFolder != null)
                {
                    if(dataFolder.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_ERR_INCORRECT_IMAGE_STATE )
                    {
                        ShowErrorMessageBox("Adding error! Internal file system is alredy built and You can not modify tree!");
                    }
                }
                else
                {
                    ShowErrorMessageBox("Adding error!");
                }
            }

			OnStopFSAddingProgress();

			// Show updated total size
			RefreshTotalSessionsSize();

			// Expand cuurent folder
			treeDiscContent.SelectedNode.Expand();

			btnClear.Enabled = true;

            btnImportTrack.Enabled = false;
		}

		private void btnAddDir_Click(object sender, EventArgs e)
		{
			// Select files for adding
			if (dlgSelectFolder.ShowDialog() != DialogResult.OK)
			{
				return;
			}

			// Create the progress window
			m_fsAddingProgressForm          = new frmFSAddingProgress(m_dataBurner);
			Enabled = false;

			AddOwnedForm(m_fsAddingProgressForm);

            // Show the progress window
			m_fsAddingProgressForm.Show();
            
            // Set the thread prameters
            // 1. Set the current folder
            m_crossThreadParam_CurFolder = null;

			if (treeDiscContent.SelectedNode.Parent == null)
			{
                m_crossThreadParam_CurFolder = m_dataBurner;
			}
			else
			{
				string pathToSearch = treeDiscContent.SelectedNode.FullPath;
                m_crossThreadParam_CurFolder = m_dataBurner.GetFileObject(pathToSearch.Substring(pathToSearch.IndexOf('\\') + 1) + '\\') as IDataFolder;
			}
            
            // 2. Set the folder path
            m_crossThreadParam_AddingFolderPath = dlgSelectFolder.SelectedPath + "\\";
            
            // 3. Clear the ret. params
            m_crossThreadParam_AddedFolder = null;

            // Start thread
			Thread fsAddingThread = new Thread(new ThreadStart(this.DoAppendDirectory));
			fsAddingThread.Start();
		}

		private void btnNewDir_Click(object sender, System.EventArgs e)
		{
			frmEditFileObject dlgEditFile = new frmEditFileObject();

			if (dlgEditFile.DoNewFolder() != DialogResult.OK)
			{
				return;
			}

			string      newFolderName = dlgEditFile.GetValue();
            
            IDataFolder parentDataFolder    = null;
			IDataFolder addedFolder;

            try
            {
                if (treeDiscContent.SelectedNode.Parent == null)
                {
                    parentDataFolder = m_dataBurner;
                }
                else
                {
                    string pathToSearch = treeDiscContent.SelectedNode.FullPath;
                    parentDataFolder    = m_dataBurner.GetFileObject(pathToSearch.Substring(pathToSearch.IndexOf('\\') + 1) + '\\') as IDataFolder;
                }

                addedFolder = parentDataFolder.CreateFolder(newFolderName) as IDataFolder;
                
                treeDiscContent.SelectedNode.Expand();
                treeDiscContent.SelectedNode = AddFileIntoTree(treeDiscContent.SelectedNode, addedFolder,false);       

				btnClear.Enabled = true;
            }
            catch
            {
                if(parentDataFolder != null)
                {
                    if(parentDataFolder.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_ERR_FILE_NAME_DUPLICATION)
                    {
                        ShowErrorMessageBox("Creating Error! Name duplication has been detected!");                        
                    }
                    else if(parentDataFolder.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_ERR_FILE_NAME_INVALID)
                    {
                        ShowErrorMessageBox("Creating Error! The specified name has incorrect chars or length of name is more than 107 chars!");
                    }
                    else if(parentDataFolder.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_ERR_INCORRECT_IMAGE_STATE )
                    {
                        ShowErrorMessageBox("Creating Error! Internal file system is alredy built and You can not modify tree!");
                    }
                }
                else
                {
                    ShowErrorMessageBox("Creating Error!");
                }
            }
		}

		private void btnRename_Click(object sender, System.EventArgs e)
		{
			string              newName;
			DialogResult        renamingResult;
			frmEditFileObject   editingForm     = new frmEditFileObject(); 
            
			if(treeDiscContent.SelectedNode.Parent == null)
			{
				return;
			}

			if (IsNodeFolder(treeDiscContent.SelectedNode))
			{
				renamingResult = editingForm.DoEditFolder(treeDiscContent.SelectedNode.Text); 
			}
			else
			{
				renamingResult = editingForm.DoEditFile(treeDiscContent.SelectedNode.Text); 
			}
            
			if(renamingResult != DialogResult.OK )
			{
				return;    
			}
			
            newName = editingForm.GetValue(); 

			// Get edited file object
			string    pathToSearch     = treeDiscContent.SelectedNode.FullPath;

			if (IsNodeFolder(treeDiscContent.SelectedNode))
			{
				pathToSearch += '\\';
			}

			IDataFile editedFileObject = m_dataBurner.GetFileObject(pathToSearch.Substring(pathToSearch.IndexOf('\\') + 1));

            try
            {
                // Rename file object
                editedFileObject.Name = newName;

                // Rename node
                treeDiscContent.SelectedNode.Text = editedFileObject.Name;
            }
            catch
            {
                if(editedFileObject != null)
                {
                    if(editedFileObject.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_ERR_FILE_NAME_DUPLICATION)
                    {
                        ShowErrorMessageBox("Renaming error! Name duplication has been detected!");                        
                    }
                    else if(editedFileObject.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_ERR_FILE_NAME_INVALID)
                    {
                        ShowErrorMessageBox("Renaming Error! The specified name has incorrect chars or length of name is more than 107 chars!");
                    }
                    else if(editedFileObject.LastErrorCode == STARBURN_RESULT_CODES.STARBURN_ERR_INCORRECT_IMAGE_STATE )
                    {
                        ShowErrorMessageBox("Renaming Error! Internal file system is alredy built and You can not modify tree!");
                    }
                }
                else
                {
                    ShowErrorMessageBox("Renaming error!");
                }
            }
		}

		private void btnRemove_Click(object sender, EventArgs e)
		{
			if (MessageBox.Show("Are you sure to delete the '" + treeDiscContent.SelectedNode.Text + "' ?", "Deleting", MessageBoxButtons.YesNo) != DialogResult.Yes)
			{
				return;
			}

			IDataFolder  parentFolder;
			IDataFile    fileToDelete;
			string       pathToSearch;

			if ((string)treeDiscContent.SelectedNode.Parent.Tag == ROOT_NODE_KEY)
			{
				parentFolder = m_dataBurner as IDataFolder ;
			}
			else
			{
				pathToSearch = treeDiscContent.SelectedNode.Parent.FullPath + '\\';
				parentFolder = m_dataBurner.GetFileObject(pathToSearch.Substring(pathToSearch.IndexOf('\\') + 1)) as IDataFolder; 
			}
			pathToSearch = treeDiscContent.SelectedNode.Text; 
            
			if(IsNodeFolder(treeDiscContent.SelectedNode))
			{
				pathToSearch +='\\';
			}

			fileToDelete = parentFolder.GetFileObject(pathToSearch);
            
            long deletingFileSize = 0;

			if(fileToDelete != null)
			{
				if (!IsNodeReadOnly(treeDiscContent.SelectedNode))
				{
                    deletingFileSize = fileToDelete.Size;
				}
			}
    
            try
            {
                // Remove file object
                parentFolder.Files.Remove(fileToDelete);
                
                // Update total size
                if (m_curSessionSize >= deletingFileSize)
                {
                    m_curSessionSize = m_curSessionSize - deletingFileSize;
                }
                else
                {
                    m_curSessionSize = 0;
                }

                // Delete node from tree
                treeDiscContent.SelectedNode.Parent.Nodes.RemoveAt(treeDiscContent.SelectedNode.Index);

                RefreshTotalSessionsSize();
            }
            catch
            {
                ShowErrorMessageBox("Removing error!");
            }
		}

		private void btnClear_Click(object sender, EventArgs e)
		{
			string qMessage = "Do you want to clear all content?'"; 
            
			if (MessageBox.Show(qMessage, "Deleting", MessageBoxButtons.YesNo) != DialogResult.Yes)
			{
				return;
			}

			IDataFolder folderToClear;

			folderToClear = m_dataBurner;
			
            m_curSessionSize = 0;
			
            folderToClear.Files.Clear();

			try
			{
				// Delete node from tree
                if(treeDiscContent.Nodes[0].Nodes != null)
                {
                    treeDiscContent.Nodes[0].Nodes.Clear();
                }

                m_isSessionImported = false;
			}
			catch
			{
				return;
			}
            
			RefreshButtonsStatus();
 
			treeDiscContent.SelectedNode = treeDiscContent.Nodes[0];
			OnNodeSelected(treeDiscContent.Nodes[0]);

            RefreshStatusImportTrack();

			RefreshTotalSessionsSize();

            m_dataBurner = null;
            m_dataBurner = new UDFDataBurner(); 
		}

		private void btnImageLabel_Click(object sender, EventArgs e)
		{
			frmEditFileObject editingForm = new frmEditFileObject();

			// Set the disc(image) Label
			DialogResult volNameResult = editingForm.DoEditDiscLabel(m_volumeName);

			if( volNameResult == DialogResult.OK )
			{
				m_volumeName = editingForm.GetValue(); 
			}
            
			RefreshDiscLabel();
		}
        
        private void frmDataBurner_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (MessageBox.Show("Do You want to exit?","Exit", MessageBoxButtons.YesNo,MessageBoxIcon.Question) == DialogResult.No)
            {
                e.Cancel = true;
            }
        }

		#endregion Events Handlers

		#region Windows Form Designer generated code
        private System.Windows.Forms.ImageList treeImages;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.ComboBox cmbDrives;
		private System.Windows.Forms.Button btnNewDir;
		private System.Windows.Forms.Button btnAddDir;
		private System.Windows.Forms.Button btnRefreshDrives;
		private System.Windows.Forms.PictureBox banner;
		private System.Windows.Forms.CheckBox chkVerifyData;
		private System.Windows.Forms.CheckBox chkTestMode;
		private System.Windows.Forms.CheckBox chkMultiSDisc;
		private System.Windows.Forms.Label lbMode;
		private System.Windows.Forms.ComboBox cmbWriteSpeed;
		private System.Windows.Forms.Label lbSpeed;  
		private System.Windows.Forms.PictureBox pictureBox2;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Button btnClose;
		private System.Windows.Forms.TreeView treeDiscContent;
		private System.Windows.Forms.Button btnAddFiles;
		private System.Windows.Forms.Button btnRename;
		private System.Windows.Forms.Button btnRemove;
		private System.Windows.Forms.Button btnClear;
		private System.Windows.Forms.GroupBox grpBootImage;
		private System.Windows.Forms.CheckBox chkBootDisc;
		private System.Windows.Forms.ComboBox cmbBootImageType;
		private System.Windows.Forms.Label lbBootImage;
		private System.Windows.Forms.Label lbBootImageType;
		private System.Windows.Forms.TextBox txtBootImageFile;
		private System.Windows.Forms.Button btnSelectBootImage;
		private System.Windows.Forms.Button btnBurn;
		private System.Windows.Forms.OpenFileDialog dlgOpenBootImageFile;
		private System.Windows.Forms.OpenFileDialog dlgSelectFiles;
		private System.Windows.Forms.FolderBrowserDialog dlgSelectFolder;
		private System.Windows.Forms.Button btnImportTrack;
		private System.ComponentModel.IContainer components = null;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Button btnImageLabel;
		internal System.Windows.Forms.ProgressBar prgrsDiscSize;
		private System.Windows.Forms.Label lbTotalSize;
		internal System.Windows.Forms.Label lbSessionSize;
		internal System.Windows.Forms.Label lbSessionSizeValue;
		internal System.Windows.Forms.Label lbFreeSpace;
		internal System.Windows.Forms.Label lbFreeSpaceValue;
		internal System.Windows.Forms.Label lbTotalSizeValue;

		protected override void Dispose(bool disposing)
		{
			if (disposing && (components != null))
			{
				components.Dispose();
			}
			base.Dispose(disposing);
		}

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(frmDataBurner));
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.label1 = new System.Windows.Forms.Label();
			this.chkVerifyData = new System.Windows.Forms.CheckBox();
			this.chkTestMode = new System.Windows.Forms.CheckBox();
			this.chkMultiSDisc = new System.Windows.Forms.CheckBox();
			this.lbMode = new System.Windows.Forms.Label();
			this.cmbWriteSpeed = new System.Windows.Forms.ComboBox();
			this.lbSpeed = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.cmbDrives = new System.Windows.Forms.ComboBox();
			this.btnRefreshDrives = new System.Windows.Forms.Button();
			this.btnClose = new System.Windows.Forms.Button();
			this.treeDiscContent = new System.Windows.Forms.TreeView();
			this.treeImages = new System.Windows.Forms.ImageList(this.components);
			this.btnAddFiles = new System.Windows.Forms.Button();
			this.btnAddDir = new System.Windows.Forms.Button();
			this.btnNewDir = new System.Windows.Forms.Button();
			this.btnRename = new System.Windows.Forms.Button();
			this.btnRemove = new System.Windows.Forms.Button();
			this.btnClear = new System.Windows.Forms.Button();
			this.grpBootImage = new System.Windows.Forms.GroupBox();
			this.btnSelectBootImage = new System.Windows.Forms.Button();
			this.txtBootImageFile = new System.Windows.Forms.TextBox();
			this.cmbBootImageType = new System.Windows.Forms.ComboBox();
			this.lbBootImage = new System.Windows.Forms.Label();
			this.lbBootImageType = new System.Windows.Forms.Label();
			this.chkBootDisc = new System.Windows.Forms.CheckBox();
			this.btnBurn = new System.Windows.Forms.Button();
			this.dlgOpenBootImageFile = new System.Windows.Forms.OpenFileDialog();
			this.dlgSelectFiles = new System.Windows.Forms.OpenFileDialog();
			this.dlgSelectFolder = new System.Windows.Forms.FolderBrowserDialog();
			this.btnImportTrack = new System.Windows.Forms.Button();
			this.pictureBox2 = new System.Windows.Forms.PictureBox();
			this.banner = new System.Windows.Forms.PictureBox();
			this.btnImageLabel = new System.Windows.Forms.Button();
			this.prgrsDiscSize = new System.Windows.Forms.ProgressBar();
			this.lbTotalSize = new System.Windows.Forms.Label();
			this.lbSessionSize = new System.Windows.Forms.Label();
			this.lbSessionSizeValue = new System.Windows.Forms.Label();
			this.lbFreeSpace = new System.Windows.Forms.Label();
			this.lbFreeSpaceValue = new System.Windows.Forms.Label();
			this.lbTotalSizeValue = new System.Windows.Forms.Label();
			this.groupBox1.SuspendLayout();
			this.grpBootImage.SuspendLayout();
			this.SuspendLayout();
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.label1);
			this.groupBox1.Controls.Add(this.chkVerifyData);
			this.groupBox1.Controls.Add(this.chkTestMode);
			this.groupBox1.Controls.Add(this.chkMultiSDisc);
			this.groupBox1.Controls.Add(this.lbMode);
			this.groupBox1.Controls.Add(this.cmbWriteSpeed);
			this.groupBox1.Controls.Add(this.lbSpeed);
			this.groupBox1.Controls.Add(this.label2);
			this.groupBox1.Controls.Add(this.cmbDrives);
			this.groupBox1.Controls.Add(this.btnRefreshDrives);
			this.groupBox1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.groupBox1.Location = new System.Drawing.Point(6, 72);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(474, 116);
			this.groupBox1.TabIndex = 2;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Burning options";
			// 
			// label1
			// 
			this.label1.Enabled = false;
			this.label1.ForeColor = System.Drawing.Color.Gray;
			this.label1.Location = new System.Drawing.Point(264, 64);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(104, 16);
			this.label1.TabIndex = 9;
			this.label1.Text = "Track-At-Once";
			// 
			// chkVerifyData
			// 
			this.chkVerifyData.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.chkVerifyData.Location = new System.Drawing.Point(326, 91);
			this.chkVerifyData.Name = "chkVerifyData";
			this.chkVerifyData.Size = new System.Drawing.Size(146, 17);
			this.chkVerifyData.TabIndex = 5;
			this.chkVerifyData.Text = "Verify data after burning";
			// 
			// chkTestMode
			// 
			this.chkTestMode.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.chkTestMode.Location = new System.Drawing.Point(159, 91);
			this.chkTestMode.Name = "chkTestMode";
			this.chkTestMode.Size = new System.Drawing.Size(177, 17);
			this.chkTestMode.TabIndex = 4;
			this.chkTestMode.Text = "Test mode (simulate burning)";
			// 
			// chkMultiSDisc
			// 
			this.chkMultiSDisc.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.chkMultiSDisc.Location = new System.Drawing.Point(15, 91);
			this.chkMultiSDisc.Name = "chkMultiSDisc";
			this.chkMultiSDisc.Size = new System.Drawing.Size(153, 17);
			this.chkMultiSDisc.TabIndex = 3;
			this.chkMultiSDisc.Text = "Create multisession disc";
			// 
			// lbMode
			// 
			this.lbMode.AutoSize = true;
			this.lbMode.Location = new System.Drawing.Point(216, 64);
			this.lbMode.Name = "lbMode";
			this.lbMode.Size = new System.Drawing.Size(38, 16);
			this.lbMode.TabIndex = 5;
			this.lbMode.Text = "Modes";
			// 
			// cmbWriteSpeed
			// 
			this.cmbWriteSpeed.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cmbWriteSpeed.ItemHeight = 13;
			this.cmbWriteSpeed.Location = new System.Drawing.Point(52, 56);
			this.cmbWriteSpeed.Name = "cmbWriteSpeed";
			this.cmbWriteSpeed.Size = new System.Drawing.Size(125, 21);
			this.cmbWriteSpeed.TabIndex = 2;
			// 
			// lbSpeed
			// 
			this.lbSpeed.AutoSize = true;
			this.lbSpeed.Location = new System.Drawing.Point(8, 60);
			this.lbSpeed.Name = "lbSpeed";
			this.lbSpeed.Size = new System.Drawing.Size(37, 16);
			this.lbSpeed.TabIndex = 5;
			this.lbSpeed.Text = "Speed";
			// 
			// label2
			// 
			this.label2.AutoSize = true;
			this.label2.Location = new System.Drawing.Point(8, 24);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(36, 16);
			this.label2.TabIndex = 5;
			this.label2.Text = "Drives";
			// 
			// cmbDrives
			// 
			this.cmbDrives.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cmbDrives.ItemHeight = 13;
			this.cmbDrives.Location = new System.Drawing.Point(52, 22);
			this.cmbDrives.Name = "cmbDrives";
			this.cmbDrives.Size = new System.Drawing.Size(256, 21);
			this.cmbDrives.TabIndex = 0;
			this.cmbDrives.SelectedIndexChanged += new System.EventHandler(this.cmbDrives_SelectedIndexChanged);
			// 
			// btnRefreshDrives
			// 
			this.btnRefreshDrives.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnRefreshDrives.Location = new System.Drawing.Point(320, 19);
			this.btnRefreshDrives.Name = "btnRefreshDrives";
			this.btnRefreshDrives.Size = new System.Drawing.Size(88, 26);
			this.btnRefreshDrives.TabIndex = 1;
			this.btnRefreshDrives.Text = "Refresh drives";
			this.btnRefreshDrives.Click += new System.EventHandler(this.btnRefreshDrives_Click);
			// 
			// btnClose
			// 
			this.btnClose.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnClose.Location = new System.Drawing.Point(384, 584);
			this.btnClose.Name = "btnClose";
			this.btnClose.Size = new System.Drawing.Size(96, 27);
			this.btnClose.TabIndex = 18;
			this.btnClose.Text = "Close";
			this.btnClose.Click += new System.EventHandler(this.btnClose_Click);
			// 
			// treeDiscContent
			// 
			this.treeDiscContent.BackColor = System.Drawing.Color.White;
			this.treeDiscContent.HideSelection = false;
			this.treeDiscContent.ImageList = this.treeImages;
			this.treeDiscContent.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.treeDiscContent.Location = new System.Drawing.Point(6, 216);
			this.treeDiscContent.Name = "treeDiscContent";
			this.treeDiscContent.Size = new System.Drawing.Size(378, 208);
			this.treeDiscContent.TabIndex = 4;
			this.treeDiscContent.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.treeDiscContent_AfterSelect);
			// 
			// treeImages
			// 
			this.treeImages.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.treeImages.ImageSize = new System.Drawing.Size(16, 16);
			this.treeImages.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("treeImages.ImageStream")));
			this.treeImages.TransparentColor = System.Drawing.Color.White;
			// 
			// btnAddFiles
			// 
			this.btnAddFiles.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnAddFiles.Location = new System.Drawing.Point(392, 216);
			this.btnAddFiles.Name = "btnAddFiles";
			this.btnAddFiles.Size = new System.Drawing.Size(86, 24);
			this.btnAddFiles.TabIndex = 7;
			this.btnAddFiles.Text = "Add file(s)";
			this.btnAddFiles.Click += new System.EventHandler(this.btnAddFiles_Click);
			// 
			// btnAddDir
			// 
			this.btnAddDir.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnAddDir.Location = new System.Drawing.Point(392, 240);
			this.btnAddDir.Name = "btnAddDir";
			this.btnAddDir.Size = new System.Drawing.Size(86, 24);
			this.btnAddDir.TabIndex = 8;
			this.btnAddDir.Text = "Add dir";
			this.btnAddDir.Click += new System.EventHandler(this.btnAddDir_Click);
			// 
			// btnNewDir
			// 
			this.btnNewDir.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnNewDir.Location = new System.Drawing.Point(392, 280);
			this.btnNewDir.Name = "btnNewDir";
			this.btnNewDir.Size = new System.Drawing.Size(86, 24);
			this.btnNewDir.TabIndex = 9;
			this.btnNewDir.Text = "New dir";
			this.btnNewDir.Click += new System.EventHandler(this.btnNewDir_Click);
			// 
			// btnRename
			// 
			this.btnRename.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnRename.Location = new System.Drawing.Point(392, 304);
			this.btnRename.Name = "btnRename";
			this.btnRename.Size = new System.Drawing.Size(86, 24);
			this.btnRename.TabIndex = 10;
			this.btnRename.Text = "Rename";
			this.btnRename.Click += new System.EventHandler(this.btnRename_Click);
			// 
			// btnRemove
			// 
			this.btnRemove.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnRemove.Location = new System.Drawing.Point(392, 336);
			this.btnRemove.Name = "btnRemove";
			this.btnRemove.Size = new System.Drawing.Size(86, 24);
			this.btnRemove.TabIndex = 11;
			this.btnRemove.Text = "Remove";
			this.btnRemove.Click += new System.EventHandler(this.btnRemove_Click);
			// 
			// btnClear
			// 
			this.btnClear.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnClear.Location = new System.Drawing.Point(392, 360);
			this.btnClear.Name = "btnClear";
			this.btnClear.Size = new System.Drawing.Size(86, 24);
			this.btnClear.TabIndex = 12;
			this.btnClear.Text = "Clear";
			this.btnClear.Click += new System.EventHandler(this.btnClear_Click);
			// 
			// grpBootImage
			// 
			this.grpBootImage.Controls.Add(this.btnSelectBootImage);
			this.grpBootImage.Controls.Add(this.txtBootImageFile);
			this.grpBootImage.Controls.Add(this.cmbBootImageType);
			this.grpBootImage.Controls.Add(this.lbBootImage);
			this.grpBootImage.Controls.Add(this.lbBootImageType);
			this.grpBootImage.Controls.Add(this.chkBootDisc);
			this.grpBootImage.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.grpBootImage.Location = new System.Drawing.Point(6, 480);
			this.grpBootImage.Name = "grpBootImage";
			this.grpBootImage.Size = new System.Drawing.Size(472, 98);
			this.grpBootImage.TabIndex = 12;
			this.grpBootImage.TabStop = false;
			this.grpBootImage.Text = "Boot image";
			// 
			// btnSelectBootImage
			// 
			this.btnSelectBootImage.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnSelectBootImage.Location = new System.Drawing.Point(431, 69);
			this.btnSelectBootImage.Name = "btnSelectBootImage";
			this.btnSelectBootImage.Size = new System.Drawing.Size(28, 21);
			this.btnSelectBootImage.TabIndex = 16;
			this.btnSelectBootImage.Text = "...";
			this.btnSelectBootImage.Click += new System.EventHandler(this.btnSelectBootImage_Click);
			// 
			// txtBootImageFile
			// 
			this.txtBootImageFile.Location = new System.Drawing.Point(74, 69);
			this.txtBootImageFile.Name = "txtBootImageFile";
			this.txtBootImageFile.ReadOnly = true;
			this.txtBootImageFile.Size = new System.Drawing.Size(353, 20);
			this.txtBootImageFile.TabIndex = 8;
			this.txtBootImageFile.TabStop = false;
			this.txtBootImageFile.Text = "";
			// 
			// cmbBootImageType
			// 
			this.cmbBootImageType.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cmbBootImageType.Location = new System.Drawing.Point(97, 42);
			this.cmbBootImageType.Name = "cmbBootImageType";
			this.cmbBootImageType.Size = new System.Drawing.Size(281, 21);
			this.cmbBootImageType.TabIndex = 15;
			// 
			// lbBootImage
			// 
			this.lbBootImage.AutoSize = true;
			this.lbBootImage.Location = new System.Drawing.Point(8, 73);
			this.lbBootImage.Name = "lbBootImage";
			this.lbBootImage.Size = new System.Drawing.Size(61, 16);
			this.lbBootImage.TabIndex = 6;
			this.lbBootImage.Text = "Boot image";
			// 
			// lbBootImageType
			// 
			this.lbBootImageType.AutoSize = true;
			this.lbBootImageType.Location = new System.Drawing.Point(8, 46);
			this.lbBootImageType.Name = "lbBootImageType";
			this.lbBootImageType.Size = new System.Drawing.Size(86, 16);
			this.lbBootImageType.TabIndex = 6;
			this.lbBootImageType.Text = "Boot image type";
			// 
			// chkBootDisc
			// 
			this.chkBootDisc.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.chkBootDisc.Location = new System.Drawing.Point(11, 19);
			this.chkBootDisc.Name = "chkBootDisc";
			this.chkBootDisc.Size = new System.Drawing.Size(101, 17);
			this.chkBootDisc.TabIndex = 14;
			this.chkBootDisc.Text = "Bootable disc";
			this.chkBootDisc.CheckedChanged += new System.EventHandler(this.chkBootDisc_CheckedChanged);
			// 
			// btnBurn
			// 
			this.btnBurn.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnBurn.Location = new System.Drawing.Point(272, 584);
			this.btnBurn.Name = "btnBurn";
			this.btnBurn.Size = new System.Drawing.Size(96, 27);
			this.btnBurn.TabIndex = 17;
			this.btnBurn.Text = "Burn";
			this.btnBurn.Click += new System.EventHandler(this.btnBurn_Click);
			// 
			// dlgOpenBootImageFile
			// 
			this.dlgOpenBootImageFile.DefaultExt = "*.img";
			this.dlgOpenBootImageFile.Filter = "Image files (*.img)|*.img|Binary files (*.bin)|*.bin|All files|*.*";
			this.dlgOpenBootImageFile.Title = "Select image file";
			// 
			// dlgSelectFiles
			// 
			this.dlgSelectFiles.Filter = "All types|*.*";
			this.dlgSelectFiles.Multiselect = true;
			this.dlgSelectFiles.RestoreDirectory = true;
			this.dlgSelectFiles.Title = "Select files";
			// 
			// dlgSelectFolder
			// 
			this.dlgSelectFolder.Description = "Select folder";
			this.dlgSelectFolder.ShowNewFolderButton = false;
			// 
			// btnImportTrack
			// 
			this.btnImportTrack.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnImportTrack.Location = new System.Drawing.Point(8, 192);
			this.btnImportTrack.Name = "btnImportTrack";
			this.btnImportTrack.Size = new System.Drawing.Size(143, 23);
			this.btnImportTrack.TabIndex = 6;
			this.btnImportTrack.Text = "Import previous session";
			this.btnImportTrack.Click += new System.EventHandler(this.btnImportTrack_Click);
			// 
			// pictureBox2
			// 
			this.pictureBox2.BackColor = System.Drawing.Color.White;
			this.pictureBox2.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox2.Image")));
			this.pictureBox2.Location = new System.Drawing.Point(427, 8);
			this.pictureBox2.Name = "pictureBox2";
			this.pictureBox2.Size = new System.Drawing.Size(49, 49);
			this.pictureBox2.TabIndex = 1;
			this.pictureBox2.TabStop = false;
			// 
			// banner
			// 
			this.banner.BackColor = System.Drawing.Color.White;
			this.banner.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.banner.Image = ((System.Drawing.Image)(resources.GetObject("banner.Image")));
			this.banner.Location = new System.Drawing.Point(0, 0);
			this.banner.Name = "banner";
			this.banner.Size = new System.Drawing.Size(488, 64);
			this.banner.TabIndex = 14;
			this.banner.TabStop = false;
			// 
			// btnImageLabel
			// 
			this.btnImageLabel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnImageLabel.Location = new System.Drawing.Point(392, 400);
			this.btnImageLabel.Name = "btnImageLabel";
			this.btnImageLabel.Size = new System.Drawing.Size(86, 24);
			this.btnImageLabel.TabIndex = 13;
			this.btnImageLabel.Text = "Disc label";
			this.btnImageLabel.Click += new System.EventHandler(this.btnImageLabel_Click);
			// 
			// prgrsDiscSize
			// 
			this.prgrsDiscSize.Location = new System.Drawing.Point(8, 432);
			this.prgrsDiscSize.Name = "prgrsDiscSize";
			this.prgrsDiscSize.Size = new System.Drawing.Size(376, 16);
			this.prgrsDiscSize.Step = 1;
			this.prgrsDiscSize.TabIndex = 23;
			// 
			// lbTotalSize
			// 
			this.lbTotalSize.AutoSize = true;
			this.lbTotalSize.Location = new System.Drawing.Point(296, 456);
			this.lbTotalSize.Name = "lbTotalSize";
			this.lbTotalSize.TabIndex = 21;
			this.lbTotalSize.Text = "Total disc capacity:";
			this.lbTotalSize.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lbSessionSize
			// 
			this.lbSessionSize.Location = new System.Drawing.Point(8, 456);
			this.lbSessionSize.Name = "lbSessionSize";
			this.lbSessionSize.Size = new System.Drawing.Size(73, 16);
			this.lbSessionSize.TabIndex = 22;
			this.lbSessionSize.Text = "Session size:";
			this.lbSessionSize.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lbSessionSizeValue
			// 
			this.lbSessionSizeValue.Location = new System.Drawing.Point(80, 456);
			this.lbSessionSizeValue.Name = "lbSessionSizeValue";
			this.lbSessionSizeValue.Size = new System.Drawing.Size(72, 16);
			this.lbSessionSizeValue.TabIndex = 20;
			this.lbSessionSizeValue.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lbFreeSpace
			// 
			this.lbFreeSpace.AutoSize = true;
			this.lbFreeSpace.Location = new System.Drawing.Point(152, 456);
			this.lbFreeSpace.Name = "lbFreeSpace";
			this.lbFreeSpace.Size = new System.Drawing.Size(64, 16);
			this.lbFreeSpace.TabIndex = 17;
			this.lbFreeSpace.Text = "Free space:";
			this.lbFreeSpace.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lbFreeSpaceValue
			// 
			this.lbFreeSpaceValue.Location = new System.Drawing.Point(224, 456);
			this.lbFreeSpaceValue.Name = "lbFreeSpaceValue";
			this.lbFreeSpaceValue.Size = new System.Drawing.Size(64, 16);
			this.lbFreeSpaceValue.TabIndex = 18;
			this.lbFreeSpaceValue.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lbTotalSizeValue
			// 
			this.lbTotalSizeValue.Location = new System.Drawing.Point(400, 456);
			this.lbTotalSizeValue.Name = "lbTotalSizeValue";
			this.lbTotalSizeValue.Size = new System.Drawing.Size(80, 16);
			this.lbTotalSizeValue.TabIndex = 19;
			this.lbTotalSizeValue.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// frmDataBurner
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(488, 622);
			this.Controls.Add(this.prgrsDiscSize);
			this.Controls.Add(this.lbTotalSize);
			this.Controls.Add(this.lbSessionSize);
			this.Controls.Add(this.lbSessionSizeValue);
			this.Controls.Add(this.lbFreeSpace);
			this.Controls.Add(this.lbFreeSpaceValue);
			this.Controls.Add(this.lbTotalSizeValue);
			this.Controls.Add(this.btnImportTrack);
			this.Controls.Add(this.grpBootImage);
			this.Controls.Add(this.btnClear);
			this.Controls.Add(this.btnRemove);
			this.Controls.Add(this.btnRename);
			this.Controls.Add(this.btnNewDir);
			this.Controls.Add(this.btnAddDir);
			this.Controls.Add(this.btnAddFiles);
			this.Controls.Add(this.treeDiscContent);
			this.Controls.Add(this.btnBurn);
			this.Controls.Add(this.btnClose);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.pictureBox2);
			this.Controls.Add(this.banner);
			this.Controls.Add(this.btnImageLabel);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.Name = "frmDataBurner";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Data Burner";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.frmDataBurner_Closing);
			this.Load += new System.EventHandler(this.frmDataBurner_Load);
			this.Closed += new System.EventHandler(this.frmDataBurner_Closed);
			this.Activated += new System.EventHandler(this.frmDataBurner_Activated);
			this.groupBox1.ResumeLayout(false);
			this.grpBootImage.ResumeLayout(false);
			this.ResumeLayout(false);

		}
        
		#endregion
        

		#region Main
		[STAThread]
		static void Main()
		{   
			Application.EnableVisualStyles();
			
			Application.DoEvents();

			Application.Run(new frmDataBurner());
            
		}
        #endregion Main
    }
}