
/*
 *
 *   Copyright (c) Rocket Division Software 2001-2010. All rights reserved.
 *   Copyright (c) StarBurn Software 2001-2010. All rights reserved.
 *
 */

using System;
using System.Threading;
using System.Windows.Forms;
 
using RocketDivision.StarBurnX;

namespace DataBurnerNS
{
    public delegate void SetBurningProgress(int percents);

    public class frmProgress : Form
    {
        #region Variables

		private		volatile	int		m_burningProgress;
		private		volatile	int		m_bufferProgress;

        private		bool				m_isCanceled;
		private		UDFDataBurner			m_dataBurner;
        
        #endregion

        #region Constructor / Destructor
        
        public frmProgress(UDFDataBurner burner)
        {
            InitializeComponent();

            m_dataBurner = burner;
            m_isCanceled = false;
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        #endregion

        #region Common
        
        public bool IsCanceled()
        {
            return m_isCanceled;
        }

        public void SetBurningProgress(int percents)
        {
            m_burningProgress = percents;
        }

        public void SetBurningBuffer(int percents)
        {
            m_bufferProgress = percents;
        }
        #endregion

        #region Handle Controls events
        
		private void progressWatcher_Tick(object sender, System.EventArgs e)
		{
			if(prgrsBurning.Value != m_burningProgress)
			{
				if(m_burningProgress >= 100)
				{
					progressWatcher.Stop();
					btnCancel.Enabled    = false; 
				}
				else
				{
					prgrsBurning.Value = m_burningProgress;
					prgrsBurning.Refresh(); 					
				}
			}

			if(prgrsBuffer.Value != m_bufferProgress)
			{
				prgrsBuffer.Value = m_bufferProgress;
				prgrsBuffer.Refresh();
			}
		}

		private void btnCancel_Click(object sender, EventArgs e)
        {
            if(m_dataBurner != null && !m_isCanceled)
            {                
                btnCancel.Enabled = false; 
                m_isCanceled = true;
                
				Thread cancelThread = new Thread(new ThreadStart(m_dataBurner.Cancel));
				cancelThread.Start();
            }
        }
        #endregion

        #region Windows Form Designer generated code
        
        private System.ComponentModel.IContainer components = null;

        private System.Windows.Forms.ProgressBar prgrsBurning;
        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.Button btnCancel;
        private System.Windows.Forms.ProgressBar prgrsBuffer;
        private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Timer progressWatcher;

        private void InitializeComponent()
        {
			this.components = new System.ComponentModel.Container();
			this.prgrsBurning = new System.Windows.Forms.ProgressBar();
			this.label1 = new System.Windows.Forms.Label();
			this.btnCancel = new System.Windows.Forms.Button();
			this.prgrsBuffer = new System.Windows.Forms.ProgressBar();
			this.label2 = new System.Windows.Forms.Label();
			this.progressWatcher = new System.Windows.Forms.Timer(this.components);
			this.SuspendLayout();
			// 
			// prgrsBurning
			// 
			this.prgrsBurning.Location = new System.Drawing.Point(12, 24);
			this.prgrsBurning.Name = "prgrsBurning";
			this.prgrsBurning.Size = new System.Drawing.Size(252, 18);
			this.prgrsBurning.Step = 5;
			this.prgrsBurning.TabIndex = 0;
			// 
			// label1
			// 
			this.label1.AutoSize = true;
			this.label1.Location = new System.Drawing.Point(12, 8);
			this.label1.Name = "label1";
			this.label1.TabIndex = 1;
			this.label1.Text = "Burning progress ..";
			// 
			// btnCancel
			// 
			this.btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnCancel.Location = new System.Drawing.Point(184, 64);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.TabIndex = 2;
			this.btnCancel.Text = "Cancel";
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// prgrsBuffer
			// 
			this.prgrsBuffer.Location = new System.Drawing.Point(12, 72);
			this.prgrsBuffer.Name = "prgrsBuffer";
			this.prgrsBuffer.Size = new System.Drawing.Size(159, 11);
			this.prgrsBuffer.TabIndex = 0;
			// 
			// label2
			// 
			this.label2.AutoSize = true;
			this.label2.Location = new System.Drawing.Point(12, 56);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(68, 16);
			this.label2.TabIndex = 1;
			this.label2.Text = "Buffer status";
			// 
			// progressWatcher
			// 
			this.progressWatcher.Enabled = true;
			this.progressWatcher.Tick += new System.EventHandler(this.progressWatcher_Tick);
			// 
			// frmProgress
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(274, 102);
			this.ControlBox = false;
			this.Controls.Add(this.prgrsBuffer);
			this.Controls.Add(this.prgrsBurning);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.label1);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "frmProgress";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Burning";
			this.TopMost = true;
			this.ResumeLayout(false);

		}

        #endregion
    }
}